package com.warewms.ams.ndc.controller;

import com.warewms.ams.ndc.dto.*;
import com.warewms.ams.ndc.service.IAmsTaskService;
import com.warewms.common.base.controller.BaseController;
import com.warewms.common.base.domain.AjaxResult;
import com.warewms.common.base.page.PageData;
import com.warewms.common.constant.Constants;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.assertj.core.util.Maps;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.util.Map;

/**
 * AGV任务Controller
 *
 * @author andy
 * @date 2022-03-10
 */
@RestController
@RequestMapping("/ams")
public class AmsTaskController extends BaseController {
    @Autowired
    private IAmsTaskService amsTaskService;

    @GetMapping("page")
    @ApiOperation("分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = Constants.PAGE, value = "当前页码，从1开始", paramType = "query", required = true, dataType = "int"),
            @ApiImplicitParam(name = Constants.LIMIT, value = "每页显示记录数", paramType = "query", required = true, dataType = "int"),
            @ApiImplicitParam(name = Constants.ORDER_FIELD, value = "排序字段", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = Constants.ORDER, value = "排序方式，可选值(asc、desc)", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "taskNo", value = "任务编号", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "aciIndex", value = "aci主键", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "stFrom", value = "起点", paramType = "query", dataType = "Long"),
            @ApiImplicitParam(name = "stTo", value = "终点", paramType = "query", dataType = "Long")
    })
    public AjaxResult page(@ApiIgnore @RequestParam Map<String, Object> params){
        PageData<AmsTaskDTO> page = amsTaskService.page(params);
        return AjaxResult.success(page);
    }

    @PostMapping(value = "/wcsTask/taskInsert")
    public AjaxResult taskInsert(@RequestBody @Valid AmsTaskInsertDTO amsTaskInsertDTO) {
        Long taskNo = amsTaskService.amsTaskInsert(amsTaskInsertDTO);
        return AjaxResult.success("任务生成成功", Maps.newHashMap("taskNo", taskNo));
    }

    @GetMapping("{id}")
    @ApiOperation("信息")
    public AjaxResult get(@PathVariable("id") Long id){
        AmsTaskDTO data = amsTaskService.get(id);
        return AjaxResult.success(data);
    }

    @GetMapping(value = "/wcsTask/getTaskInfo")
    public AjaxResult getTaskInfo(@RequestParam String businessNo) {
        AmsTaskInfoDTO AmsTaskInfoDTO = amsTaskService.getAmsTaskInfo(businessNo);
        return AjaxResult.success(AmsTaskInfoDTO);
    }

    @PostMapping(value = "/wcsTask/taskCancel")
    public AjaxResult taskCancel(@RequestBody @Valid AmsTaskCancelDTO amsTaskCancelDTO) {
        amsTaskService.cancelAmsTask(amsTaskCancelDTO);
        return AjaxResult.success();
    }

    @PostMapping(value = "/cancelAmsTaskToWms")
    public AjaxResult cancelAmsTaskToWms(@RequestBody @Valid AmsTaskCancelToWmsDTO amsTaskCancelToWmsDTO) {
        amsTaskService.cancelAmsTaskToWms(amsTaskCancelToWmsDTO);
        return AjaxResult.success();
    }

    @PostMapping(value = "/completeAmsTaskToWms")
    public AjaxResult completeAmsTaskToWms(@RequestParam Long id) {
        amsTaskService.completeAmsTaskToWms(id);
        return AjaxResult.success();
    }

}
