package com.warewms.ams.ndc.controller;


import com.warewms.ams.ndc.domain.BaseLocationInfo;
import com.warewms.ams.ndc.dto.BaseLocationInfoDTO;
import com.warewms.ams.ndc.dto.BaseLocationInfoInsertDTO;
import com.warewms.ams.ndc.service.IBaseLocationInfoService;
import com.warewms.common.annotation.Log;
import com.warewms.common.base.controller.BaseController;
import com.warewms.common.base.domain.AjaxResult;
import com.warewms.common.base.page.PageData;
import com.warewms.common.constant.Constants;
import com.warewms.common.enums.BusinessType;
import com.warewms.common.utils.poi.ExcelUtil;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.Map;

/**
 * 库位信息Controller
 *
 * @author andy
 * @date 2022-02-18
 */
@RestController
@RequestMapping("/base/locationInfo")
public class BaseLocationInfoController extends BaseController {
    @Autowired
    private IBaseLocationInfoService baseLocationInfoService;

    @GetMapping("page")
    @ApiOperation("分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = Constants.PAGE, value = "当前页码，从1开始", paramType = "query", required = true, dataType = "int"),
            @ApiImplicitParam(name = Constants.LIMIT, value = "每页显示记录数", paramType = "query", required = true, dataType = "int"),
            @ApiImplicitParam(name = Constants.ORDER_FIELD, value = "排序字段", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = Constants.ORDER, value = "排序方式，可选值(asc、desc)", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "zoneId", value = "区域id", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "locationNo", value = "库位编码", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "rowNo", value = "巷道", paramType = "query", dataType = "Long"),
            @ApiImplicitParam(name = "agvStation", value = "agv点位", paramType = "query", dataType = "Long")
    })
    public AjaxResult page(@ApiIgnore @RequestParam Map<String, Object> params){
        PageData<BaseLocationInfoDTO> page = baseLocationInfoService.page(params);
        return AjaxResult.success(page);
    }

    @PostMapping("/save")
    @ApiOperation("保存")
    public AjaxResult save(@RequestBody BaseLocationInfoInsertDTO dto){

        baseLocationInfoService.insertBaseLocationInfo(dto);

        return AjaxResult.success();
    }

    /**
     * 获取库位信息详细信息
     */
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return AjaxResult.success(baseLocationInfoService.get(id));
    }

    /**
     * 修改库位信息
     */
    @Log(title = "库位信息", businessType = BusinessType.UPDATE)
    @PutMapping("/edit")
    public AjaxResult edit(@RequestBody BaseLocationInfoDTO baseLocationInfoDTO) {
        baseLocationInfoService.update(baseLocationInfoDTO);
        return AjaxResult.success();
    }

    /**
     * 删除库位信息
     */
    @Log(title = "库位信息", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(baseLocationInfoService.deleteBatchIds(Arrays.asList(ids)));
    }

    @PostMapping("/importTemplate")
    public void importTemplate(HttpServletResponse response)
    {
        ExcelUtil<BaseLocationInfo> util = new ExcelUtil<>(BaseLocationInfo.class);
        util.importTemplateExcel(response, "库位信息");
    }
}
