package com.warewms.ams.ndc.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.warewms.ams.common.AmsConstant;
import com.warewms.ams.ndc.domain.BaseLocationInfo;
import com.warewms.ams.ndc.dto.BaseLocationInfoDTO;
import com.warewms.ams.ndc.dto.BaseLocationInfoInsertDTO;
import com.warewms.ams.ndc.mapper.BaseLocationInfoMapper;
import com.warewms.ams.ndc.service.IBaseLocationInfoService;
import com.warewms.common.utils.ConvertUtils;
import com.warewms.common.utils.redis.RedisCache;
import com.warewms.framework.service.impl.CrudServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.util.List;
import java.util.Map;

/**
 * 库位信息Service业务层处理
 *
 * @author andy
 * @date 2022-02-18
 */
@Slf4j
@Service
public class BaseLocationInfoServiceImpl extends CrudServiceImpl<BaseLocationInfoMapper, BaseLocationInfo, BaseLocationInfoDTO>  implements IBaseLocationInfoService {

    @Autowired
    private BaseLocationInfoMapper baseLocationInfoMapper;

    @Autowired
    private RedisCache redisCache;

    /**
     * 项目启动时，初始化参数到缓存
     */
    @PostConstruct
    public void init() {
        loadingLocationCache();
    }

    /**
     * 加载参数缓存数据
     */
    public void loadingLocationCache() {
        List<BaseLocationInfo> baseLocationInfoList = baseLocationInfoMapper.selectList(Wrappers.emptyWrapper());
        for (BaseLocationInfo baseLocationInfo : baseLocationInfoList) {
            redisCache.setCacheObject(AmsConstant.WMS_CORRESPOND_AGV_STATION_KEY.concat(baseLocationInfo.getLocationNo()), baseLocationInfo.getAgvStation());
            redisCache.setCacheObject(AmsConstant.LOCATION_ROW_KEY.concat(baseLocationInfo.getLocationNo()), baseLocationInfo.getRowNo());
            redisCache.setCacheObject(AmsConstant.AGV_TO_PILLING_CAR_KEY.concat(baseLocationInfo.getAgvStation().toString()), baseLocationInfo.getLocationNo());
        }
    }

    @Override
    public List<BaseLocationInfo> queryBaseLocationInfoList(Integer zoneId, String rowNo) {
        return queryBaseLocationInfoList(zoneId, null, rowNo);
    }

    private List<BaseLocationInfo> queryBaseLocationInfoList(Integer zoneId, String locationNo, String rowNo) {
        return baseDao.selectList(Wrappers.<BaseLocationInfo>lambdaQuery()
                .eq(ObjectUtil.isNotNull(zoneId), BaseLocationInfo::getZoneId, zoneId)
                .eq(StringUtils.isNotBlank(locationNo), BaseLocationInfo::getLocationNo, locationNo)
                .eq(StringUtils.isNotBlank(rowNo), BaseLocationInfo::getRowNo, rowNo));
    }

    @Override
    public void insertBaseLocationInfo(BaseLocationInfoInsertDTO baseLocationInfoInsertDTO){
        BaseLocationInfo baseLocationInfo = ConvertUtils.sourceToTarget(baseLocationInfoInsertDTO, BaseLocationInfo.class);
        baseLocationInfo.setRowIndex(1l);
        baseLocationInfo.setColNo("1");
        baseLocationInfo.setColIndex(1l);
        baseLocationInfo.setShiftNo("1");
        baseLocationInfo.setShiftIndex(1l);
        baseLocationInfo.setStockStatus("00");
        baseLocationInfo.setColNo("1");
        baseDao.insert(baseLocationInfo);
    }

    @Override
    public QueryWrapper<BaseLocationInfo> getWrapper(Map<String, Object> params) {
        QueryWrapper<BaseLocationInfo> queryWrapper = new QueryWrapper<>();
        Long zoneId = StringUtils.isNotBlank((String) params.get("zoneId")) ? Long.parseLong((String) params.get("zoneId")) : null;
        String locationNo = (String) params.get("locationNo");
        String rowNo = (String) params.get("rowNo");
        Long agvStation = StringUtils.isNotBlank((String) params.get("agvStation")) ? Long.parseLong((String) params.get("agvStation")) : null;
        queryWrapper.lambda()
                .eq(ObjectUtil.isNotNull(zoneId), BaseLocationInfo::getZoneId, zoneId)
                .eq(StringUtils.isNotBlank(locationNo), BaseLocationInfo::getLocationNo, locationNo)
                .eq(StringUtils.isNotBlank(rowNo), BaseLocationInfo::getRowNo, rowNo)
                .eq(ObjectUtil.isNotNull(agvStation), BaseLocationInfo::getAgvStation, agvStation);
        return queryWrapper;
    }
}
